
"""
risk_manage_engine/filter.py

리스크 점수(risk_score)를 기반으로 투자 적격 종목만 필터링하는 모듈입니다.

- 기준치 이상의 종목만 `tradable = True` 표시
- 분석 결과와 리스크 평가 결과를 결합해 출력
"""

import pandas as pd

def apply_risk_filter(predictions: pd.DataFrame, risk_scores: dict, threshold: int = 70) -> pd.DataFrame:
    """
    예측 결과와 리스크 점수를 결합하여 필터링 결과 반환

    Parameters:
        predictions (pd.DataFrame): symbol, prediction, probability 등의 예측 결과
        risk_scores (dict): {symbol: score} 형태의 리스크 점수
        threshold (int): 투자 적격 기준치 (기본 70)

    Returns:
        pd.DataFrame: 종목 + 예측 결과 + risk_score + tradable 여부 포함
    """
    predictions = predictions.copy()
    predictions['risk_score'] = predictions['symbol'].map(risk_scores)
    predictions['tradable'] = predictions['risk_score'] >= threshold
    return predictions

def filter_by_risk(df: pd.DataFrame, threshold: int = 60) -> pd.DataFrame:
    """
    risk_score 컬럼을 기반으로 필터링

    Parameters:
        df (pd.DataFrame): risk_score 컬럼이 포함된 데이터프레임
        threshold (int): 필터링 기준점 (기본 60)

    Returns:
        pd.DataFrame: risk_score >= threshold 인 종목만 반환
    """
    return df[df["risk_score"] >= threshold].reset_index(drop=True)
